@@ -2,8 +2,8 @@
 these files remotely from the OPEN-ADAS website.
 '''
 
-import urllib
 import shutil,os
+import requests
 
 # location of the "adas_data" directory relative to this script:
 adas_data_dir = os.path.dirname(os.path.realpath(__file__))+os.sep+'adas_data'+os.sep
@@ -59,11 +59,8 @@ def get_adas_file_loc(filename, filetype='adf11'):
         return adas_data_dir+filetype+os.sep+filename
 
     elif os.path.exists(filename):
-        # check if user actually gave a complete filepath. Don't copy the file from the original location
-        #filepath = filename
-        #filename = filepath.split('/')[-1]
-        #shutil.copyfile(filepath, adas_data_dir+filetype+os.sep+filename)
-        return filename #adas_data_dir+filetype+os.sep+filename
+        # user gave a complete filepath. Don't copy the file from the original location
+        return filename
     
     elif 'AURORA_ADAS_DIR' in os.environ:
         loc = os.environ['AURORA_ADAS_DIR']+os.sep+filetype+os.sep+filename
@@ -101,10 +98,16 @@ def fetch_adf11_file(filename, loc):
     url = 'https://open.adas.ac.uk/download/adf11/'
     str1 =  filename.split('_')[0]
     
-    local_filename,headers = urllib.request.urlretrieve(
-        url+str1+'/'+filename,
-        filename = loc
-    )
+    r = requests.get(url+str1+'/'+filename)
+
+    if len(r.text)<1000:
+        # OPEN-ADAS reports short URL error text rather than an error code
+        raise ValueError(f'Could not fetch file {filename} from ADAS!')
+
+    with open(loc, 'wb') as f:
+        f.write(r.content)
+
+
 
 
 
@@ -133,12 +136,18 @@ def fetch_adf15_file(filename, loc):
         # patterns may be different, attempt simple guess:
         filename_mod = filename.split('_')[0]+'/'+filename.replace('#','][')
 
-    local_filename,headers = urllib.request.urlretrieve(
-        url+'/'+filename_mod,
-        filename=loc
-    )
+    r = requests.get(url+'/'+filename_mod)
+
+    if len(r.text)<1000:
+        # OPEN-ADAS reports short URL error text rather than an error code
+        raise ValueError(f'Could not fetch file {filename_mod} from ADAS!')
+
+    with open(loc, 'wb') as f:
+        f.write(r.content)
+
     
 
+
 def adas_files_dict():
     '''Selections for ADAS files for Aurora runs and radiation calculations.
     This function can be called to fetch a set of default files, which can then be modified (e.g. to 
@@ -276,27 +285,27 @@ def adas_files_dict():
     files["Na"] = {}   #10
     files["Na"]['acd'] = "acd85_ne.dat"
     files["Na"]['scd'] = "scd85_ne.dat"
-    files["Na"]['prb'] = ""
-    files["Na"]['plt'] = ""
-    files["Na"]['ccd'] = ""
-    files["Na"]['pls'] = ""
-    files["Na"]['prs'] = ""
-    files["Na"]['fis'] = ""
-    files["Na"]['brs'] = ""
-    files["Na"]["pbs"] = ""
-    files["Na"]['prc'] = ""
+    #files["Na"]['prb'] = ""
+    #files["Na"]['plt'] = ""
+    #files["Na"]['ccd'] = ""
+    #files["Na"]['pls'] = ""
+    #files["Na"]['prs'] = ""
+    #files["Na"]['fis'] = ""
+    #files["Na"]['brs'] = ""
+    #files["Na"]["pbs"] = ""
+    #files["Na"]['prc'] = ""
     files["Mg"] = {}   #12
     files["Mg"]['acd'] = "acd85_mg.dat"
     files["Mg"]['scd'] = "scd85_mg.dat"
-    files["Mg"]['prb'] = ""
-    files["Mg"]['plt'] = ""
-    files["Mg"]['ccd'] = ""
-    files["Mg"]['pls'] = ""
-    files["Mg"]['prs'] = ""
-    files["Mg"]['fis'] = ""
-    files["Mg"]['brs'] = ""
-    files["Mg"]["pbs"] = ""
-    files["Mg"]['prc'] = ""   
+    #files["Mg"]['prb'] = ""
+    #files["Mg"]['plt'] = ""
+    #files["Mg"]['ccd'] = ""
+    #files["Mg"]['pls'] = ""
+    #files["Mg"]['prs'] = ""
+    #files["Mg"]['fis'] = ""
+    #files["Mg"]['brs'] = ""
+    #files["Mg"]["pbs"] = ""
+    #files["Mg"]['prc'] = ""   
     files["Al"] = {}    #13
     files["Al"]['acd'] = "acd89_al.dat"
     files["Al"]['scd'] = "scd89_al.dat"
