@@ -257,8 +257,11 @@ def get_atom_data(imp, files=['acd','scd']):
         as attributes atom_data[key].logNe, atom_data[key].logT, atom_data[key].data
     '''
 
-    # default files dictionary
-    all_files = adas_files.adas_files_dict()[imp] # all default files for a given species
+    try:
+        # default files dictionary
+        all_files = adas_files.adas_files_dict()[imp] # all default files for a given species
+    except KeyError:
+        raise KeyError(f'ADAS data not available for ion {imp}!')
     
     if isinstance(files, dict):
         # user provided files choices as a dictionary
@@ -271,12 +274,16 @@ def get_atom_data(imp, files=['acd','scd']):
     for key in keys_to_delete:
         del all_files[key]
 
+    for filecheck in files:
+        if filecheck not in all_files:
+            raise ValueError(f'Could not fetch {imp} {filecheck.upper()} file!')
+    
     atom_data = {}
     for filetype in all_files:
 
         # find location of required ADF11 file
         fileloc = adas_files.get_adas_file_loc(all_files[filetype],filetype='adf11')
-
+    
         # load specific file and add it to output dictionary
         res = adas_file(fileloc)
         atom_data[filetype] = res.logNe, res.logT, res.data
@@ -293,11 +300,10 @@ def null_space(A):
     return Q, s[-2]  # matrix is singular, so last index is ~0
 
 
-def get_frac_abundances(atom_data, ne_cm3, Te_eV=None, n0_by_ne=1e-5,
-                        include_cx=False, ne_tau=np.inf,
-                        plot=True, ax = None, rho = None, rho_lbl=None,ls='-'):
+def get_frac_abundances(atom_data, ne_cm3, Te_eV=None, n0_by_ne=0.0,
+                        ne_tau=np.inf, plot=True, ax = None, rho = None, rho_lbl=None,ls='-'):
     r'''Calculate fractional abundances from ionization and recombination equilibrium.
-    If include_cx=True, radiative recombination and thermal charge exchange are summed.
+    If n0_by_ne is not 0, radiative recombination and thermal charge exchange are summed.
 
     This method can work with ne,Te and n0_by_ne arrays of arbitrary dimension, but plotting 
     is only supported in 1D (defaults to flattened arrays).
@@ -312,9 +318,7 @@ def get_frac_abundances(atom_data, ne_cm3, Te_eV=None, n0_by_ne=1e-5,
         Electron temperature in units of eV. If left to None, the Te grid given in the 
         atomic data is used.
     n0_by_ne: float or array, optional
-        Ratio of background neutral hydrogen to electron density, used if include_cx=True. 
-    include_cx : bool
-        If True, charge exchange with background thermal neutrals is included. 
+        Ratio of background neutral hydrogen to electron density. If not 0, CX is considered. 
     ne_tau : float, opt
         Value of electron density in :math:`m^{-3}\cdot s` :math:`\times` particle residence time. 
         This is a scalar value that can be used to model the effect of transport on ionization equilibrium. 
@@ -338,23 +342,18 @@ def get_frac_abundances(atom_data, ne_cm3, Te_eV=None, n0_by_ne=1e-5,
         rate coefficients are given.
     fz : array, (space,nZ)
         Fractional abundances across the same grid used by the input ne,Te values. 
-    rate_coeffs : array, (space, nZ)
-        Rate coefficients in units of [:math:`s^{-1}`]. 
+
     '''
     # if input arrays are multi-dimensional, flatten them here and restructure at the end
-    if not isinstance(ne_cm3,float):
-        _ne = ne_cm3.flatten()
-    else:
-        _ne = copy.deepcopy(ne_cm3)
-    if not isinstance(Te_eV,float): 
-        _Te = Te_eV.flatten()
-    else:
-        _Te = copy.deepcopy(Te_eV)
-    if not isinstance(n0_by_ne,float):
-        _n0_by_ne = n0_by_ne.flatten()
-    else:
-        _n0_by_ne = copy.deepcopy(n0_by_ne)
+    _ne = np.array(ne_cm3).flatten()
+    _Te = np.array(Te_eV).flatten()
+    _n0_by_ne = np.array(n0_by_ne).flatten()
 
+    include_cx = False if (isinstance(n0_by_ne,(int,float)) and n0_by_ne==0.0) else True
+
+    #from IPython import embed
+    #embed()
+    
     logTe, logS,logR,logcx = get_cs_balance_terms(
         atom_data, _ne, _Te, maxTe=10e3, include_cx=include_cx)
     
@@ -362,25 +361,12 @@ def get_frac_abundances(atom_data, ne_cm3, Te_eV=None, n0_by_ne=1e-5,
         # Get an effective recombination rate by summing radiative & CX recombination rates
         logR= np.logaddexp(logR,np.log(_n0_by_ne)[:,None] +logcx)
 
-    # numerical method that calculates also rate_coeffs
-    nion = logR.shape[1]
-    fz  = np.zeros((logTe.size,nion+1))
-    rate_coeffs = np.zeros(logTe.size)
+    # simplest method to compute fractional abundances
+    rate_ratio = np.hstack((np.zeros_like(logTe)[:, None], logS - logR))
+    fz = np.exp(np.cumsum(rate_ratio, axis=1))
+    fz /= fz.sum(1)[:, None]
 
-    for it,t in enumerate(logTe):
-
-        A = (
-              - np.diag(np.r_[np.exp(logS[it]), 0] + np.r_[0, np.exp(logR[it])] + 1e6 / ne_tau)
-              + np.diag(np.exp(logS[it]), -1)
-              + np.diag(np.exp(logR[it]), 1)
-              )
-
-        N,rate_coeffs[it] = null_space(A)
-        fz[it] = N/np.sum(N)
-
-    rate_coeffs*=(_ne * 1e-6)
-
-    if plot:
+    if plot and np.size(_ne)>1:
         # plot fractional abundances (only 1D)
         if ax is None:
             fig,axx = plt.subplots()
@@ -421,12 +407,12 @@ def get_frac_abundances(atom_data, ne_cm3, Te_eV=None, n0_by_ne=1e-5,
         axx.set_ylim(0,1.05)
         axx.set_xlim(x[0],x[-1])
 
-    # re-structure to original array dimensions
-    logTe = logTe.reshape(ne_cm3.shape)
-    fz = fz.reshape(*ne_cm3.shape, fz.shape[1])
-    rate_coeffs = rate_coeffs.reshape(*ne_cm3.shape)
+    if np.size(ne_cm3)>1:
+        # re-structure to original array dimensions
+        logTe = logTe.reshape(np.array(ne_cm3).shape)
+        fz = fz.reshape(*np.array(ne_cm3).shape, fz.shape[1])
     
-    return logTe, fz, rate_coeffs
+    return logTe, fz
 
 
 
@@ -493,29 +479,98 @@ def get_cs_balance_terms(atom_data, ne_cm3=5e13, Te_eV=None, maxTe=10e3, include
 
 
 
-def plot_relax_time(logTe, rate_coeff, ax = None):
-    ''' Plot relaxation time of the ionization equilibrium corresponding
-    to the inverse of the given rate coefficients.
+def get_atomic_relax_time(atom_data, ne_cm3, Te_eV=None, n0_by_ne=0.0,
+                        ne_tau=np.inf, plot=True, ax = None, ls='-'):
+    r'''Obtain the relaxation time of the ionization equilibrium for a given atomic species.
+
+    If n0_by_ne is not 0, thermal charge exchange is added to radiative and dielectronic recombination.
+    This function can work with ne,Te and n0_by_ne arrays of arbitrary dimension.
+    It uses a matrix SVD approach in order to find the relaxation rates, as opposed to the simpler
+    approach of :py:func:`~aurora.get_frac_abundances`, but fractional abundances produced by the two
+    methods should be the same.
 
     Parameters
     ----------
-    logTe : array (nr,)
-        log-10 of Te [eV], on an arbitrary grid (same as other arguments, but not
-        necessarily radial)
-    rate_coeff : array (nr,)
-        Rate coefficients from ionization balance, as returned by the :py:fun:`~aurora.atomic.get_frac_abundances` function.
-        N.B.: these rate coefficients will depend also on electron density, which does affect relaxation times. 
-    ax : matplotlib axes instance, optional
-        If provided, plot relaxation times on these axes.    
-    '''
-    if ax is None:
-        ax = plt.subplot(111)
+    atom_data : dictionary of atomic ADAS files (only acd, scd are required; ccd is 
+        necessary only if n0_by_ne is not 0).
+    ne_cm3 : float or array
+        Electron density in units of :math:`cm^{-3}`.
+    Te_eV : float or array, optional
+        Electron temperature in units of eV. If left to None, the Te grid given in the 
+        atomic data is used.
+    n0_by_ne: float or array, optional
+        Ratio of background neutral hydrogen to electron density. If set to 0, CX is not 
+        considered.
+    ne_tau : float, opt
+        Value of electron density in :math:`m^{-3}\cdot s` :math:`\times` particle residence time. 
+        This is a scalar value that can be used to model the effect of transport on ionization equilibrium. 
+        Setting ne_tau=np.inf (default) corresponds to no effect from transport. 
+    plot : bool, optional
+        If True, the atomic relaxation time is plotted as a function of Te. Default is True.
+    ax : matplotlib.pyplot Axes instance
+        Axes on which to plot if plot=True. If False, new axes are created.
+    ls : str, optional
+        Line style for plots. Continuous lines are used by default. 
 
-    ax.loglog(10**logTe,1e3/rate_coeff,'b' )
-    ax.set_xlim(10**logTe[0],10**logTe[-1])
-    ax.grid('on')
-    ax.set_xlabel('T$_e$ [eV]')
-    ax.set_ylabel(r'$\tau_\mathrm{relax}$ [ms]')
+    Returns
+    -------
+    logTe : array
+        log10 of electron temperatures as a function of which the fractional abundances and
+        rate coefficients are given.
+    fz : array, (space,nZ)
+        Fractional abundances across the same grid used by the input ne,Te values. 
+    rate_coeffs : array, (space, nZ)
+        Rate coefficients in units of [:math:`s^{-1}`]. 
+    '''
+    # if input arrays are multi-dimensional, flatten them here and restructure at the end
+    _ne = np.array(ne_cm3).flatten()
+    _Te = np.array(Te_eV).flatten()
+    _n0_by_ne = np.array(n0_by_ne).flatten()
+
+    include_cx = False if (isinstance(n0_by_ne,(int,float)) and n0_by_ne==0.0) else True
+
+    logTe, logS,logR,logcx = get_cs_balance_terms(
+        atom_data, _ne, _Te, maxTe=10e3, include_cx=include_cx)
+    
+    if include_cx:
+        # Get an effective recombination rate by summing radiative & CX recombination rates
+        logR= np.logaddexp(logR,np.log(_n0_by_ne)[:,None] +logcx)
+
+    # numerical method that calculates also rate_coeffs
+    nion = logR.shape[1]
+    fz  = np.zeros((logTe.size,nion+1))
+    rate_coeffs = np.zeros(logTe.size)
+
+    for it,t in enumerate(logTe):
+        A = (
+              - np.diag(np.r_[np.exp(logS[it]), 0] + np.r_[0, np.exp(logR[it])] + 1e6 / ne_tau)
+              + np.diag(np.exp(logS[it]), -1)
+              + np.diag(np.exp(logR[it]), 1)
+              )
+
+        N,rate_coeffs[it] = null_space(A)
+        fz[it] = N/np.sum(N)
+
+    rate_coeffs*=(_ne * 1e-6)
+
+    if np.size(ne_cm3)>1:
+        # re-structure to original array dimensions
+        logTe = logTe.reshape(np.array(ne_cm3).shape)
+        fz = fz.reshape(*np.array(ne_cm3).shape, fz.shape[1])
+        rate_coeffs = rate_coeffs.reshape(*np.array(ne_cm3).shape)
+
+    if plot:
+        # Now plot relaxation times
+        if ax is None:
+            fig,ax = plt.subplots()
+            
+        ax.loglog(10**logTe,1e3/rate_coeffs,'b' )
+        ax.set_xlim(10**logTe[0],10**logTe[-1])
+        ax.grid('on')
+        ax.set_xlabel('T$_e$ [eV]')
+        ax.set_ylabel(r'$\tau_\mathrm{relax}$ [ms]')
+        
+    return logTe, fz, rate_coeffs
 
 
 
@@ -788,20 +843,6 @@ def impurity_brems(nz, ne, Te):
        
         
 
-def balance(logTe_val, cs, n0_by_ne, logTe_, S,R,cx):
-    '''Evaluate balance of effective ionization, recombination and charge exchange at a given temperature. '''
-
-    a = R +n0_by_ne * cx
-    SS_0 = 10**(interp1d(logTe_, np.log10(S[cs-1,:]), kind='cubic', bounds_error=False)(logTe_val))
-    aa_0 = 10**(interp1d(logTe_, np.log10(a[cs-1,:]), kind='cubic', bounds_error=False)(logTe_val))
-    SS_1 = 10**(interp1d(logTe_, np.log10(S[cs,:]), kind='cubic', bounds_error=False)(logTe_val))
-    aa_1 = 10**(interp1d(logTe_, np.log10(a[cs,:]), kind='cubic', bounds_error=False)(logTe_val))
-
-    val = SS_0 - SS_1 - aa_0 + aa_1
-    return val*1e20 # get this to large-ish units to avoid tolerance issues due to small powers of 10
-
-
-
 
 def plot_norm_ion_freq(S_z, q_prof, R_prof, imp_A, Ti_prof,
                        nz_profs=None, rhop=None, plot=True, eps_prof=None):
@@ -886,3 +927,167 @@ def plot_norm_ion_freq(S_z, q_prof, R_prof, imp_A, Ti_prof,
         ax.set_xlim([0,1])
 
 
+
+def read_adf12(filename, block, Ebeam, ne_cm3, Ti_eV, zeff):
+    '''Read charge exchange effective emission coefficients from ADAS ADF12 files.
+
+    See https://open.adas.ac.uk/adf12 for details and to download files.
+
+    Parameters
+    ----------
+    filename : str
+        adf12 file name/path
+    block : int
+        Source block selected
+    Ebeam : float
+        Energy of the neutral beam population of interest, in units of :math:`eV/amu`.
+    ne_cm3 : float or 1D array
+        Electron densities at which to evaluate coefficients, in units of :math:`cm^{-3}`.
+    Ti_eV : float or 1D array
+        Bulk ion temperature at which to evaluate coefficients, in units of :math:`eV`.
+    Zeff : float or 1D array
+        Effective background charge.
+
+    Returns
+    -------
+    float or 1D array :
+        Interpolated coefficients, units of :math:`cm^3/s`.
+
+    '''
+    with open(filename, 'r') as f:
+        nlines = int(f.readline())
+
+        for iline in range(block):
+            cer_line = {}
+            params = []
+            first_line = '0'
+            while not first_line[0].isalpha():
+                first_line = f.readline()
+
+            cer_line['header'] = first_line
+            cer_line['qefref'] = np.float(f.readline()[:63].replace('D', 'e'))
+            cer_line['parmref'] = np.float_(f.readline()[:63].replace('D', 'e').split())
+            cer_line['nparmsc'] = np.int_(f.readline()[:63].split())
+
+            for ipar, npar in enumerate(cer_line['nparmsc']):
+                for q in range(2):
+                    data = []
+                    while npar > len(data):
+                        line = f.readline()
+                        if len(line) > 63:
+                            name = line[63:].strip().lower()
+                            cer_line[name] = []
+                            if q == 0:
+                                params.append(name)
+
+                        values = np.float_(line[:63].replace('D', 'E').split())
+                        values = values[values > 0]
+                        if not len(values):
+                            continue
+                        data += values.tolist()
+                    cer_line[name] = data
+
+    # interpolate in logspace
+    lqefref = np.log(cer_line['qefref'])
+    
+    lnq = np.zeros(np.broadcast(Ebeam, ne_cm3, Ti_eV, zeff).shape)
+    lnq += lqefref * (1 - 4)
+    lnq += np.interp(np.log(Ti_eV), np.log(cer_line['tiev']), np.log(cer_line['qtiev']))
+    lnq += np.interp(np.log(ne_cm3), np.log(cer_line['densi']), np.log(cer_line['qdensi']))
+    lnq += np.interp(np.log(Ebeam), np.log(cer_line['ener']), np.log(cer_line['qener']))
+    lnq += np.interp(np.log(zeff), np.log(cer_line['zeff']), np.log(cer_line['qzeff']))
+    
+    return np.exp(lnq)
+
+
+        
+def read_adf21(filename, Ebeam, ne_cm3, Te_eV):
+    '''Read ADAS ADF21 or ADF22 files.
+
+    ADF21 files contain effective beam stopping/excitation coefficients.
+    ADF22 contain effective beam emission/population coefficients.
+
+    See https://open.adas.ac.uk/adf21 and https://open.adas.ac.uk/adf22 for details
+    and to download files.
+
+    Parameters
+    ----------
+    filename : str
+        adf21 or adf22 file name/path
+    Ebeam : float
+        Energy of the neutral beam, in units of :math:`eV/amu`.
+    ne_cm3 : float or 1D array
+        Electron densities at which to evaluate coefficients.
+    Te_eV : float or 1D array
+        Electron temperature at which to evaluate coefficients.
+
+    Returns
+    -------
+    float or 1D array :
+        Interpolated coefficients. For ADF21 files, these have units of :math:`cm^3/s`for ADF21 files.
+        For ADF22, they correspond to n=2 fractional abundances.
+    '''
+    
+    with open(filename, 'r') as f:
+        line = f.readline()
+        ref = float(line.split()[1].split('=')[1])
+        f.readline()
+        line = f.readline()
+        nE, nne, Teref = line.split()
+        nE, nne = int(nE), int(nne)
+        Teref = float(Teref.split('=')[1])
+        f.readline()
+
+        E = []
+        while len(E) < nE:
+            line = f.readline()
+            E.extend([float(f) for f in line.split()])
+        E = np.array(E)
+
+        ne = []
+        while len(ne) < nne:
+            line = f.readline()
+            ne.extend([float(f) for f in line.split()])
+        ne = np.array(ne)
+        f.readline()
+
+        Q2 = []
+        while len(Q2) < nne * nE:
+            line = f.readline()
+            Q2.extend([float(f) for f in line.split()])
+        Q2 = np.reshape(Q2, (nne, nE))
+
+        f.readline()
+        line = f.readline()
+        nTe, Eref, Neref = line.split()
+        nTe, Eref, Neref = int(nTe), float(Eref.split('=')[1]), float(Neref.split('=')[1])
+
+        f.readline()
+
+        Te = []
+        while len(Te) < nTe:
+            line = f.readline()
+            Te.extend([float(f) for f in line.split()])
+        Te = np.array(Te)
+
+        f.readline()
+
+        Q1 = []
+        while len(Q1) < nTe:
+            line = f.readline()
+            Q1.extend([float(f) for f in line.split()])
+        Q1 = np.array(Q1)
+
+    # clip data in available range to avoid extrapolation
+    Ebeam = np.clip(Ebeam, *E[[0, -1]])
+    ne_cm3 = np.clip(ne_cm3, *ne[[0, -1]])
+    Te_eV = np.clip(Te_eV, *Te[[0, -1]])
+
+    lref = np.log(ref)
+
+    # interpolate on the requested values
+    RectInt1 = interp.interp1d(np.log(Te), np.log(Q1) - lref, assume_sorted=True, kind='quadratic')
+    RectInt2 = interp.RectBivariateSpline(np.log(ne), np.log(E), np.log(Q2) - lref, kx=2, ky=2)
+
+    adf = RectInt1(np.log(Te_eV)) + RectInt2.ev(np.log(ne_cm3), np.log(Ebeam))
+    return np.exp(adf + lref)
