@@ -16,7 +16,7 @@ from . import plot_tools
 def compute_rad(imp, nz, ne, Te,
                 n0 = None, Ti = None, ni = None, adas_files_sub = {},
                 prad_flag=False, sxr_flag=False,
-                thermal_cx_rad_flag=False, spectral_brem_flag=False, ):
+                thermal_cx_rad_flag=False, spectral_brem_flag=False):
     '''Calculate radiation terms corresponding to a simulation result. The nz,ne,Te,n0,Ti,ni arrays
     are normally assumed to be given as a function of (time,nZ,space), but time and space may 
     be substituted by other coordinates (e.g. R,Z)
@@ -117,10 +117,10 @@ def compute_rad(imp, nz, ne, Te,
     # calculate total radiation
     if prad_flag:
 
-        atom_data = atomic.get_atom_data(imp, files = {'plt': adas_files_sub['plt'] if 'plt' in adas_files_sub else None})
+        atom_data = atomic.get_atom_data(imp, files = {'plt': adas_files_sub.get('plt',None)})
         pltt = atomic.interp_atom_prof(atom_data['plt'],logne,logTe) # W
 
-        atom_data = atomic.get_atom_data(imp, files = {'prb': adas_files_sub['prb'] if 'prb' in adas_files_sub else None})
+        atom_data = atomic.get_atom_data(imp, files = {'prb': adas_files_sub.get('prb',None)})
         prb = atomic.interp_atom_prof(atom_data['prb'],logne,logTe) # W
 
         # line radiation for each charge state
@@ -150,25 +150,26 @@ def compute_rad(imp, nz, ne, Te,
             assert n0.ndim==2 and Ti.ndim==2
             
             logTi = np.log10(Ti)
-            
-            # thermal CX radiation to total recombination and continuum radiation terms:
-            atom_data = atomic.get_atom_data(imp, files = {'prc': adas_files_sub['prc'] if 'prc' in adas_files_sub else None})
+            try:
+                # thermal CX radiation to total recombination and continuum radiation terms:
+                atom_data = atomic.get_atom_data(imp, files = {'prc': adas_files_sub.get('prc',None)})
 
-            # prc has weak dependence on density, so no difference between using ni or ne
-            prc = atomic.interp_atom_prof(atom_data['prc'],logne,logTi,x_multiply=False) # W
+                # prc has weak dependence on density, so no difference between using ni or ne
+                prc = atomic.interp_atom_prof(atom_data['prc'],logne,logTi,x_multiply=False) # W
 
-            # broadcast n0 to dimensions (nt,nZ,nr):
-            res['thermal_cx_cont_rad'] = nz[:,1:] * n0[:,None] * prc
+                # broadcast n0 to dimensions (nt,nZ,nr):
+                res['thermal_cx_cont_rad'] = nz[:,1:] * n0[:,None] * prc
 
-            # add to total unfiltered radiation:
-            res['tot'] += res['thermal_cx_cont_rad'].sum(1)
+                # add to total unfiltered radiation:
+                res['tot'] += res['thermal_cx_cont_rad'].sum(1)
+            except:
+                res['thermal_cx_cont_rad'] = 0
                        
     if sxr_flag: # SXR-filtered radiation (spectral range depends on filter used for files)
-
-        atom_data = atomic.get_atom_data(imp, files = {'pls': adas_files_sub['pls'] if 'pls' in adas_files_sub else None})
+        atom_data = atomic.get_atom_data(imp, files = {'pls': adas_files_sub.get('pls',None)})
         pls = atomic.interp_atom_prof(atom_data['pls'],logne,logTe) # W
 
-        atom_data = atomic.get_atom_data(imp, files = {'prs': adas_files_sub['prs'] if 'prs' in adas_files_sub else None})
+        atom_data = atomic.get_atom_data(imp, files = {'prs': adas_files_sub.get('prs', None)})
         prs = atomic.interp_atom_prof(atom_data['prs'],logne,logTe) # W
 
         # SXR line radiation for each charge state
@@ -179,10 +180,10 @@ def compute_rad(imp, nz, ne, Te,
 
         try:
             # impurity bremsstrahlung in SXR range -- already included in 'sxr_cont_rad'
-            atom_data = atomic.get_atom_data(imp, files = {'pbs': adas_files_sub['pbs'] if 'pbs' in adas_files_sub else None})
+            atom_data = atomic.get_atom_data(imp, files = {'pbs': adas_files_sub.get('pbs',None)})
             pbs = atomic.interp_atom_prof(atom_data['pbs'],logne,logTe) # W
             res['sxr_brems'] = nz[:,1:] * pbs 
-        except IndexError:
+        except:
             # pbs file not available by default for this ion. Users may specify it in adas_files_sub
             pass
         
@@ -192,7 +193,7 @@ def compute_rad(imp, nz, ne, Te,
         
     if spectral_brem_flag:  # spectral bremsstrahlung (i.e. brems at a specific wavelength)
 
-        atom_data = atomic.get_atom_data(imp, files = {'brs': adas_files_sub['brs'] if 'brs' in adas_files_sub else None})
+        atom_data = atomic.get_atom_data(imp, files = {'brs': adas_files_sub.get('brs',None)})
         x,y,tab = atom_data['brs']
         brs = atomic.interp_atom_prof((x,y,tab.T),None,logTe) # W
 
@@ -263,7 +264,7 @@ def sync_rad(B_T, ne_cm3, Te_eV, r_min_cm, R_maj_cm):
 def radiation_model(imp,rhop, ne_cm3, Te_eV, vol,
                     adas_files_sub={}, n0_cm3=None, Ti_eV=None, nz_cm3=None, frac=None, plot=False):
     '''Model radiation from a fixed-impurity-fraction model or from detailed impurity density
-    profiles for the chosen ion. This method acts as a wrapper for :py:method:compute_rad(), 
+    profiles for the chosen ion. This method acts as a wrapper for :py:func:`~aurora.compute_rad`,
     calculating radiation terms over the radius and integrated over the plasma cross section. 
 
     Parameters
@@ -332,10 +333,10 @@ def radiation_model(imp,rhop, ne_cm3, Te_eV, vol,
     if nz_cm3 is None:
         # load ionization and recombination rates
         atom_files = {}
-        atom_files['acd'] = adas_files_sub['acd'] if 'acd' in adas_files_sub else adas_files.adas_files_dict()[imp]['acd']
-        atom_files['scd'] = adas_files_sub['scd'] if 'scd' in adas_files_sub else adas_files.adas_files_dict()[imp]['scd']
+        atom_files['acd'] = adas_files_sub.get('acd', adas_files.adas_files_dict()[imp]['acd'])
+        atom_files['scd'] = adas_files_sub.get('scd', adas_files.adas_files_dict()[imp]['scd'])
         if n0_cm3 is not None:
-            atom_files['ccd'] = adas_files_sub['ccd'] if 'ccd' in adas_files_sub else adas_files.adas_files_dict()[imp]['ccd']
+            atom_files['ccd'] = adas_files_sub.get('ccd',adas_files.adas_files_dict()[imp]['ccd'])
         
         # now load ionization and recombination rates
         atom_data = atomic.get_atom_data(imp,files=atom_files)
@@ -366,7 +367,7 @@ def radiation_model(imp,rhop, ne_cm3, Te_eV, vol,
                       Ti=Te_eV[None,:] if Ti_eV is None else Ti_eV[None,:],
                       adas_files_sub=adas_files_sub,
                       prad_flag=True, sxr_flag=False,
-                      thermal_cx_rad_flag=False if n0_cm3 is None else True,
+                      thermal_cx_rad_flag= n0_cm3 is not None,
                       spectral_brem_flag=False)
 
     # radiation terms -- converted from W/cm^3 to W/m^3
@@ -568,9 +569,10 @@ def read_adf15(path, order=1, plot_lines=[], ax=None, plot_3d=False):
     log10pec_dict = {}
 
     for i in range(0, num_lines):
-        
-        if '----' in lines[0]: 
-            _ = lines.pop(0) # separator may exist before each transition
+
+        while 'isel' not in lines[0].lower():
+            # eliminate variable number of label lines at the top
+            _ = lines.pop(0)
 
         # Get the wavelength, number of densities and number of temperatures
         # from the first line of the entry:
@@ -628,9 +630,17 @@ def read_adf15(path, order=1, plot_lines=[], ax=None, plot_3d=False):
             # attempt to report unknown rate type -- this should be fairly robust
             rate_type = l.replace(' ','').lower().split('type=')[1].split('/')[0]
 
+        # sometimes multiple lines of the same rate_type can be listed at the same wavelength
+        # separate them here by 1e-6 A
+        while True:
+            if lam in log10pec_dict and rate_type in log10pec_dict[lam]:
+                lam += 1e-6
+            else:
+                break
+
         # create dictionary with keys for each wavelength:
         if lam not in log10pec_dict:
-            log10pec_dict[lam] = {}                
+            log10pec_dict[lam] = {}
 
         # add a key to the log10pec_dict[lam] dictionary for each type of rate: recom, excit or chexc
         # interpolate PEC on log dens,temp scales
@@ -699,11 +709,13 @@ def _plot_pec(dens, temp, PEC, PEC_eval, lam,cs,rate_type, ax=None, plot_3d=Fals
     else:
         # plot in 2D
         labels = ['{:.0e}'.format(ne)+r' $cm^{-3}$' for ne in dens] #ne_eval]
-            
-        for ine in np.arange(PEC.shape[0]):
-            l, = ax1.plot(temp, PEC_eval[ine,:], label=labels[ine])
-            ax1.plot(temp, PEC[ine,:], color=l.get_color(), marker='o', mfc=l.get_color(), ms=5.)
 
+        ls_cycle = plot_tools.get_ls_cycle()
+        for ine in np.arange(PEC.shape[0]):
+            ls = next(ls_cycle)
+            l, = ax1.plot(temp, PEC_eval[ine,:], ls, label=labels[ine])
+            #ax1.plot(temp, PEC[ine,:], color=l.get_color(), marker='o', mfc=l.get_color(), ms=5.)
+            ax1.plot(temp, PEC[ine,:], ls, marker='o', mfc=l.get_color(), ms=5.)
         ax1.set_xlabel(r'$T_e$ [eV]')
         ax1.set_ylabel('PEC [photons $\cdot cm^3/s$]')
         ax1.set_yscale('log')
@@ -715,7 +727,7 @@ def _plot_pec(dens, temp, PEC, PEC_eval, lam,cs,rate_type, ax=None, plot_3d=Fals
 
 def get_local_spectrum(adf15_filepath, ion, ne_cm3, Te_eV,
                        Ti_eV=None, n0_cm3=0.0, ion_exc_rec_dens=None, dlam_A=0.0,
-                       plot=True, ax=None, plot_spec_tot=True, no_leg=False, plot_all_lines=False):
+                       plot_spec_tot=True, plot_all_lines=False, no_leg=False, ax=None):
     r'''Plot spectrum based on the lines contained in an ADAS ADF15 file
     at specific values of electron density and temperature. Charge state densities
     can be given explicitely, or alternatively charge state fractions will be automatically 
@@ -746,18 +758,16 @@ def get_local_spectrum(adf15_filepath, ion, ne_cm3, Te_eV,
         Doppler shift in A. This can either be a scalar or an array of the same shape as the 
         output wavelength array. For the latter option, it is recommended to call this function
         twice to find the wave_final_A array first.         
-    plot : bool
-        If True, all spectral emission components are plotted.
-    ax : matplotlib Axes instance
-        Axes to plot on if plot=True. If left to None, a new figure is created.
     plot_spec_tot : bool
         If True, plot total spectrum (sum over all components) from given ADF15 file. 
-    no_leg : bool
-        If True, no plot legend is shown. Default is False, i.e. show legend.
     plot_all_lines : bool
         If True, plot all individual lines, rather than just the profiles due to different atomic processes.
         If more than 50 lines are included, a down-selection is automatically made to avoid excessive
         memory consumption.
+    no_leg : bool
+        If True, no plot legend is shown. Default is False, i.e. show legend.
+    ax : matplotlib Axes instance
+        Axes to plot on if plot=True. If left to None, a new figure is created.
 
     Returns
     -------
@@ -842,26 +852,18 @@ def get_local_spectrum(adf15_filepath, ion, ne_cm3, Te_eV,
             atom_data, np.array([ne_cm3,]), np.array([Te_eV,]),
             n0_by_ne=np.array([n0_cm3/ne_cm3,]), include_cx=True, plot=False)
         ion_exc_rec_dens = [fz[0][-4], fz[0][-3], fz[0][-2]] # Li-like, He-like, H-like
-
-    wave_A = np.zeros((len(list(log10pec_dict.keys()))))
-    pec_ion = np.zeros((len(list(log10pec_dict.keys()))))
-    pec_exc = np.zeros((len(list(log10pec_dict.keys()))))
-    pec_rr = np.zeros((len(list(log10pec_dict.keys()))))
-    pec_cx = np.zeros((len(list(log10pec_dict.keys()))))
-    pec_dr = np.zeros((len(list(log10pec_dict.keys()))))
-    for ii,lam in enumerate(log10pec_dict):
-        wave_A[ii] = lam
-        if 'ioniz' in log10pec_dict[lam]:
-            pec_ion[ii] = 10**log10pec_dict[lam]['ioniz'].ev(np.log10(ne_cm3),np.log10(Te_eV))
-        if 'excit' in log10pec_dict[lam]:
-            pec_exc[ii] = 10**log10pec_dict[lam]['excit'].ev(np.log10(ne_cm3),np.log10(Te_eV))
-        if 'recom' in log10pec_dict[lam]:
-            pec_rr[ii] = 10**log10pec_dict[lam]['recom'].ev(np.log10(ne_cm3),np.log10(Te_eV))
-        if 'chexc' in log10pec_dict[lam]:
-            pec_cx[ii] = 10**log10pec_dict[lam]['checx'].ev(np.log10(ne_cm3),np.log10(Te_eV))
-        if 'drsat' in log10pec_dict[lam]:
-            pec_dr[ii] = 10**log10pec_dict[lam]['drsat'].ev(np.log10(ne_cm3),np.log10(Te_eV))
     
+    nlines = len(list(log10pec_dict.keys()))
+    wave_A = np.zeros(nlines)
+ 
+    pec = {}
+    for typ in ['ioniz', 'excit', 'recom','chexc', 'drsat' ]:
+        pec[typ] = np.zeros(nlines)
+        for ii,lam in enumerate(log10pec_dict):
+            wave_A[ii] = lam
+            if typ in log10pec_dict[lam]:
+                pec[typ][ii] = 10**log10pec_dict[lam][typ].ev(np.log10(ne_cm3),np.log10(Te_eV))
+        
     # Doppler broadening
     mass = constants.m_p * ion_A
     dnu_g = np.sqrt(2.*(Ti_eV*constants.e)/mass)*(constants.c/wave_A)/constants.c
@@ -879,66 +881,38 @@ def get_local_spectrum(adf15_filepath, ion, ne_cm3, Te_eV,
     
     wave_final_A = np.linspace(np.min(lams_profs_A), np.max(lams_profs_A), 100000)
     
-    if plot and ax is None:
+    if (plot_all_lines or plot_spec_tot) and ax is None:
         fig,ax = plt.subplots()
 
-    many_lines=False
-    if len(wave_A)>50:
-        many_lines=True
-
     # contributions to spectrum
+    source = {'ioniz': 0, 'excit': 1, 'recom': 2, 'chexc': 2, 'drsat': 2}
     spec_ion = np.zeros_like(wave_final_A)
-    for ii in np.arange(lams_profs_A.shape[0]):
-        comp_ion = interp1d(lams_profs_A[ii,:], ne_cm3*ion_exc_rec_dens[0]*pec_ion[ii]*theta[ii,:],
-                               bounds_error=False, fill_value=0.0)(wave_final_A)
-        if plot_all_lines and many_lines and pec_ion[ii]>np.max(pec_ion)/100:
-            ax.plot(wave_final_A+dlam_A, comp_ion, c='r')
-        spec_ion += comp_ion
-
-    spec_exc = np.zeros_like(wave_final_A)
-    for ii in np.arange(lams_profs_A.shape[0]):
-        comp_exc = interp1d(lams_profs_A[ii,:], ne_cm3*ion_exc_rec_dens[1]*pec_exc[ii]*theta[ii,:],
-                               bounds_error=False, fill_value=0.0)(wave_final_A)
-        if plot_all_lines and many_lines and pec_exc[ii]>np.max(pec_exc)/100:
-            ax.plot(wave_final_A+dlam_A, comp_exc, c='b')
-        spec_exc += comp_exc
-
-    spec_rr = np.zeros_like(wave_final_A)
-    for ii in np.arange(lams_profs_A.shape[0]):
-        comp_rr = interp1d(lams_profs_A[ii,:], ne_cm3*ion_exc_rec_dens[2]*pec_rr[ii]*theta[ii,:],
-                               bounds_error=False, fill_value=0.0)(wave_final_A)
-        if plot_all_lines and many_lines and pec_rr[ii]>np.max(pec_rr)/100:
-            ax.plot(wave_final_A+dlam_A, comp_rr, c='g')
-        spec_rr += comp_rr
-
-    spec_dr = np.zeros_like(wave_final_A)
-    for ii in np.arange(lams_profs_A.shape[0]):
-        comp_dr = interp1d(lams_profs_A[ii,:], ne_cm3*ion_exc_rec_dens[2]*pec_dr[ii]*theta[ii,:],
-                               bounds_error=False, fill_value=0.0)(wave_final_A)
-        if plot_all_lines and many_lines and pec_dr[ii]>np.max(pec_dr)/100:
-            ax.plot(wave_final_A+dlam_A, comp_dr, c='g')
-        spec_dr += comp_dr
-
-    spec_cx = np.zeros_like(wave_final_A)
-    for ii in np.arange(lams_profs_A.shape[0]):
-        comp_cx = interp1d(lams_profs_A[ii,:], n0_cm3*ion_exc_rec_dens[2]*pec_cx[ii]*theta[ii,:],
-                               bounds_error=False, fill_value=0.0)(wave_final_A)
-        if plot_all_lines and many_lines and pec_cx[ii]>np.max(pec_cx)/100:
-            ax.plot(wave_final_A+dlam_A, comp_cx, c='g')
-        spec_cx += comp_cx
-
-    spec_tot = spec_ion+spec_exc+spec_rr+spec_dr+spec_cx
+    spec = {}
+    spec_tot = np.zeros_like(wave_final_A)
+    for typ,c in zip(['ioniz', 'excit', 'recom','drsat','chexc'],['r','b','g','m','c']):
+        spec[typ] = np.zeros_like(wave_final_A)
+        for ii in np.arange(lams_profs_A.shape[0]):
+            comp = interp1d(lams_profs_A[ii,:], ne_cm3*ion_exc_rec_dens[source[typ]]*pec[typ][ii]*theta[ii,:],
+                                bounds_error=False, fill_value=0.0)(wave_final_A)
+            if plot_all_lines and pec[typ][ii]>np.max(pec[typ])/1000:
+                ax.plot(wave_final_A+dlam_A, comp, c=c)
+            spec[typ] += comp
+        spec_tot += spec[typ]
+  
     
-    if plot:
+    if plot_spec_tot:
         # plot contributions from different processes
-        ax.plot(wave_final_A+dlam_A, spec_ion, c='r', label='' if no_leg else 'ionization')
-        ax.plot(wave_final_A+dlam_A, spec_exc, c='b', label='' if no_leg else 'excitation')
-        ax.plot(wave_final_A+dlam_A, spec_rr, c='g', label='' if no_leg else 'radiative recomb')
-        ax.plot(wave_final_A+dlam_A, spec_dr, c='m', label='' if no_leg else 'dielectronic recomb')
-        ax.plot(wave_final_A+dlam_A, spec_cx, c='c', label='' if no_leg else 'charge exchange recomb')
-        if plot_spec_tot:
-            ax.plot(wave_final_A+dlam_A, spec_tot, c='k', label='' if no_leg else 'total')
+        ax.plot(wave_final_A+dlam_A, spec['ioniz'], c='r', ls='--', label='' if no_leg else 'ionization')
+        ax.plot(wave_final_A+dlam_A, spec['excit'], c='b', ls='--', label='' if no_leg else 'excitation')
+        ax.plot(wave_final_A+dlam_A, spec['recom'], c='g', ls='--', label='' if no_leg else 'radiative recomb')
+        ax.plot(wave_final_A+dlam_A, spec['drsat'], c='m', ls='--', label='' if no_leg else 'dielectronic recomb')
+        ax.plot(wave_final_A+dlam_A, spec['chexc'], c='c', ls='--', label='' if no_leg else 'charge exchange recomb')
 
+        # total envelope
+        ax.plot(wave_final_A+dlam_A, spec_tot, c='k', ls='--', label='' if no_leg else 'total')
+
+
+    if plot_all_lines or plot_spec_tot:
         if not no_leg: ax.legend(loc='best').set_draggable(True)
         ax.set_xlabel(r'$\lambda$ [$\AA$]')
         ax.set_ylabel(r'$\epsilon$ [A.U.]')
@@ -946,7 +920,7 @@ def get_local_spectrum(adf15_filepath, ion, ne_cm3, Te_eV,
         ax=None
 
     # return Doppler-shifted wavelength if dlam_A was given as non-zero
-    return wave_final_A+dlam_A, spec_ion, spec_exc, spec_rr, spec_dr, spec_cx, ax
+    return wave_final_A+dlam_A, spec['ioniz'], spec['excit'], spec['recom'], spec['drsat'], spec['chexc'], ax
 
 
 
@@ -996,7 +970,7 @@ def get_cooling_factors(imp, ne_cm3, Te_eV, n0_cm3=0.0,
 
     '''
     files = ['scd','acd']
-    if n0_cm3 is not 0.0: files+=['ccd']
+    if n0_cm3 != 0.0: files+=['ccd']
     atom_data_eq = atomic.get_atom_data(imp,files)
 
     logTe, fz, rates = atomic.get_frac_abundances(atom_data_eq, ne_cm3, Te_eV,plot=False,
@@ -1041,6 +1015,126 @@ def get_cooling_factors(imp, ne_cm3, Te_eV, n0_cm3=0.0,
 
 
 
+
+def adf15_line_identification(pec_files, lines=None, Te_eV = 1e3, ne_cm3=5e13, mult=[]):
+    '''Display all photon emissivity coefficients from the given list of ADF15 files and (optionally) compare to a set
+    of chosen wavelengths, given in units of Angstrom.
+
+    Parameters
+    -----------------
+    pec_files : str or list of str
+        Path to a single ADF15 file or a list of files.
+    lines : dict, list or 1D array
+        Lines to overplot with the loaded PECs to consider overlap within spectrum. This argument may be a dictionary, with
+        keys corresponding to line names and values corresponding to wavelengths (in units of Angstrom). If provided as a 
+        list or array, this is assumed to contain only wavelengths in Angstrom.
+    Te_eV : float
+        Single value of electron temperature at which PECs should be evaluated [:math:`eV`].
+    ne_cm3 : float
+        Single value of electron density at which PECs should be evaluated [:math:`cm^{-3}`].
+    mult : list or array
+        Multiplier to apply to lines from each PEC file. This could be used for example to rescale the results of
+        multiple ADF15 files by the expected fractional abundance or density of each element/charge state.
+
+    Notes
+    --------
+    To attempt identification of spectral lines, one can load a set of ADF15 files, calculate approximate fractional
+    abundances at equilibrium and overplot expected emissivities in a few steps::
+
+    >>> pec_files = ['mypecs1','mypecs2','mypecs3']
+    >>> Te_eV=500; ne_cm3=5e13; ion='Ar'   # examples
+    >>> atom_data = aurora.atomic.get_atom_data(ion,['scd','acd'])
+    >>> logTe, fz = aurora.atomic.get_frac_abundances(atom_data, ne_cm3, Te_eV, plot=False)
+    >>> mult = [fz[0,10], fz[0,11], fz[0,12]] # to select charge states 11+, 12+ and 13+, for example
+    >>> adf15_line_identification(pec_files, Te_eV=Te_eV, ne_cm3=ne_cm3, mult=mult)
+    '''
+
+    fig = plt.figure(figsize=(10,7))
+    a_id = plt.subplot2grid((10,10),(0,0),rowspan = 2, colspan = 8, fig=fig)
+    ax = plt.subplot2grid((10,10),(2,0),rowspan = 8, colspan = 8, fig=fig, sharex=a_id)
+    a_legend = plt.subplot2grid((10,10),(0,8),rowspan = 10, colspan = 2, fig=fig) 
+    a_id.axis('off')
+    a_legend.axis('off')
+    
+    ymin = np.inf
+    ymax= -np.inf
+
+    if isinstance(pec_files,str):
+        pec_files = [pec_files,]
+
+    if len(mult) and len(pec_files)!=len(mult):
+        raise ValueError('Different number of ADF15 files and multipliers detected!')
+    
+    cols = iter(plt.cm.rainbow(np.linspace(0,1,len(pec_files))))
+
+    lams = []
+    for pp, pec_file in enumerate(pec_files):
+        
+        # load single PEC file from given list
+        log10_pecs = read_adf15(pec_file)
+
+        _mult = mult[pp] if len(mult) else 1.
+
+        c = next(cols)
+
+        # now plot all ionization-, excitation- and recombination-driven components
+        for lam, log10pec_interps in log10_pecs.items():
+
+            if 'ioniz' in log10pec_interps:
+                pec_ion = _mult * 10 ** log10pec_interps['ioniz'](np.log10(ne_cm3), np.log10(Te_eV))[0, 0]
+                if pec_ion > 1e-20:  # plot only stronger lines
+                    ymin = min(pec_ion, ymin)
+                    ymax = max(pec_ion, ymax)
+                    ax.semilogy([lam, lam], [1e-70, pec_ion], 'o-.', c=c)
+            if 'excit' in log10pec_interps:
+                pec_exc = _mult * 10 ** log10pec_interps['excit'](np.log10(ne_cm3), np.log10(Te_eV))[0, 0]
+                if pec_exc > 1e-20:  # plot only stronger lines
+                    ymin = min(pec_exc, ymin)
+                    ymax = max(pec_exc, ymax)
+                    ax.semilogy([lam, lam], [1e-70, pec_exc], 'o-', c=c)
+            if 'recom' in log10pec_interps:
+                pec_rec = _mult * 10 ** log10pec_interps['recom'](np.log10(ne_cm3), np.log10(Te_eV))[0, 0]
+                if pec_rec > 1e-20:
+                    ymin = min(pec_rec, ymin)
+                    ymax = max(pec_rec, ymax)
+                    ax.semilogy([lam, lam], [1e-70, pec_rec], 'o--', c=c)
+
+        lams += log10_pecs.keys()
+        if len(pec_files)>1:
+            a_legend.plot([],[], c=c, label=pec_file.split('/')[-1])
+
+    
+    ax.set_ylim(ymin, ymax*2)
+    ax.set_xlim(min(lams) / 1.5, max(lams) * 1.5)
+
+    ax.set_xlabel(r'$\lambda$ [$\AA$]')
+    ax.set_ylabel('PEC [phot $\cdot$ cm$^3$/s]')
+    a_id.set_title(r'$T_e$ = %d eV, $n_e$ = %.2e cm$^{-3}$' % (Te_eV, ne_cm3))
+
+    # plot location of certain lines of interest
+    if lines is not None:
+        a_legend.axvline(np.nan, label='Input lines')
+        if isinstance(lines, dict):
+            for name, wvl in lines.items():
+                ax.axvline(wvl, c='k', lw=2.0)
+                a_id.text(wvl, 0.1, name, rotation=90) #, clip_on=True)
+        else:
+            for i, wvl in enumerate(lines):
+                ax.axvline(wvl, c='k', lw=2.0)
+                a_id.text(wvl, 0.1, str(i), rotation=90) #, clip_on=True)
+                
+    a_legend.plot([], [], 'o-.', label='PEC ionization')
+    a_legend.plot([], [], 'o-', label='PEC excitation')
+    a_legend.plot([], [], 'o--', label='PEC recombination')
+    
+    if len(pec_files)==1:
+        # show path of single file that was passed
+        a_legend.text(1.05, -0.05, pec_files[0], rotation=90, transform=a_legend.transAxes)
+        
+    a_legend.legend(loc='best').set_draggable(True)
+
+
+
 def adf04_files():
     '''Collection of trust-worthy ADAS ADF04 files. 
     This function will be moved and expanded in ColRadPy in the near future. 
