@@ -97,7 +97,7 @@ class solps_case:
             _P_idxs = np.concatenate((_P_idxs, np.arange(self.unit_p+1,3*self.unit_p+1)))  # core/open SOL
             self.P_idxs = np.concatenate((_P_idxs, np.arange(3*self.unit_p+1,4*self.unit_p+2)))  # outer PFR
 
-            
+
         elif form=='full':
             from omfit_classes import omfit_solps # import omfit_solps here to avoid issues with docs and packaging
             self.b2fstate = omfit_solps.OMFITsolps(self.path+os.sep+self.solps_run+os.sep+'b2fstate')
@@ -363,8 +363,7 @@ class solps_case:
             try:
                 out[key] = np.array(out[key]).reshape(*fort44_info[key][1],order='F')
             except:
-                # could not parse this quantity. Check why!
-                print(f'Variable {key} in fort.44 could not be parsed!')
+                print(f'Variable {key} in fort.44 could not be parsed, likely to be new in SOLPS-ITER.')
 
         return out
 
@@ -537,9 +536,8 @@ class solps_case:
         ax.set_title(label)
         ax.set_xlabel('R [m]')
         ax.set_ylabel('Z [m]')
-        plt.grid(True)
+        #plt.grid(True)
         ax.axis('scaled')
-
         
 
 
@@ -822,13 +820,13 @@ def get_fort44_info(NDX,NDY,NATM,NMOL,NION,NSTRA,NCL,NPLS,NSTS,NLIM):
         'emissmol': [r'$H_\alpha$ emissivity due to molecules and molecular ions ($photons m^{-2} s^{-1}$)', (NDX,NDY)],
         'srcml': [r'Molecule particle source (A)', (NDX,NDY,NMOL)],
         'edissml': [r'Energy spent for dissociating hydrogenic molecules (W)', (NDX,NDY,NMOL)],
-        'wldnek': [r'Heat transferred by neutrals (W), total over strata', (NLIM+NSTS)],
-        'wldnep': [r'Potential energy released by neutrals (W), total over strata', (NLIM+NSTS)],
+        'wldnek': [r'Heat transferred by neutrals (W), total over strata', (NLIM+NSTS,)],
+        'wldnep': [r'Potential energy released by neutrals (W), total over strata', (NLIM+NSTS,)],
         'wldna': [r'Flux of atoms impinging on surface (A), total over strata', (NLIM+NSTS,NATM)],
         'ewlda': [r'Average energy of impinging atoms on surface (eV), total over strata', (NLIM+NSTS,NATM)],
         'wldnm': [r'Flux of molecules impinging on surface (A), total over strata', (NLIM+NSTS,NMOL)],
         'ewldm': [r'Average energy of impinging molecules on surface (eV), total over strata', (NLIM+NSTS,NMOL)],
-        'p1,p2': [r'Endpoints of surface (X and Y coordinates, in m), total over strata', (NLIM)],
+        'p1,p2': [r'Endpoints of surface (X and Y coordinates, in m), total over strata', (NLIM,)],
         'wldra': [r'Flux of reflected atoms from surface (A), total over strata', (NLIM+NSTS,NATM)],
         'wldrm': [r'Flux of reflected molecules from surface (A), total over strata', (NLIM+NSTS,NMOL)],
     }
@@ -899,9 +897,10 @@ def get_fort44_info(NDX,NDY,NATM,NMOL,NION,NSTRA,NCL,NPLS,NSTS,NLIM):
             'wldspta_res': [r'Flux of sputtered wall material per atom (A)', (NCL, NATM)],
             'wldsptm_res': [r'Flux of sputtered wall material per molecule (A)', (NCL, NMOL)],
             'wlpump_res(A)': [r'Pumped flux per atom (A)', (NCL, NATM)],
-            'wlpump_res(M)': [r' Pumped flux per molecule (A)', (NCL, NMOL)],
-            'wlpump_res(I)': [r' Pumped flux per test ion (A)', (NCL, NION)],
-            'wlpump_res(P)': [r' Pumped flux per plasma ion (A)', (NCL, NPLS)],
+            'wlpump_res(M)': [r'Pumped flux per molecule (A)', (NCL, NMOL)],
+            'wlpump_res(I)': [r'Pumped flux per test ion (A)', (NCL, NION)],
+            'wlpump_res(P)': [r'Pumped flux per plasma ion (A)', (NCL, NPLS)],
+            'ewldt_res': [r'Total wall power loading from Eirene particles', (NCL,)],
             'pdena_int': [r'Integral number of atoms over the entire Eirene computational grid', (NATM, NSTRA+1)],
             'pdenm_int': [r'Integral number of molecules over the entire Eirene computational grid', (NMOL, NSTRA+1)],
             'pdeni_int': [r'Integral number of test ions over the entire Eirene computational grid', (NION, NSTRA+1)],
@@ -943,11 +942,11 @@ def get_fort46_info(NTRII,NATM,NMOL,NION):
         'VZDENA': [r'Z-directed momentum density carried by atoms ($g*cm^{-2}*s^{-1}$)', (NTRII,NATM)],
         'VZDENM': [r'Z-directed momentum density carried by molecules ($g*cm^{-2}*s^{-1}$)', (NTRII,NMOL)],
         'VZDENI': [r'Z-directed momentum density carried by test ions ($g*cm^{-2}*s^{-1}$)', (NTRII,NION)],
-        'VOLUMES': [r'Triangle volumes ($cm^{-3}$)', (NTRII)],
-        'PUX': [r'X-component of the poloidal unit vector at the triangle center', (NTRII)],
-        'PUY': [r'Y-component of the poloidal unit vector at the triangle center', (NTRII)],
-        'PVX': [r'X-component of the radial unit vector at the triangle center', (NTRII)],
-        'PVY': [r'Y-component of the radial unit vector at the triangle center', (NTRII)],
+        'VOLUMES': [r'Triangle volumes ($cm^{-3}$)', (NTRII,)],
+        'PUX': [r'X-component of the poloidal unit vector at the triangle center', (NTRII,)],
+        'PUY': [r'Y-component of the poloidal unit vector at the triangle center', (NTRII,)],
+        'PVX': [r'X-component of the radial unit vector at the triangle center', (NTRII,)],
+        'PVY': [r'Y-component of the radial unit vector at the triangle center', (NTRII,)],
     }
     
-    return fort46_info
\ No newline at end of file
+    return fort46_info
